 /****************************************************************************
 *  pptppro.h     : $Revision:   1.16  $
 *  Modified      : $Date:   May 13 1996 14:48:44  $
 *  By            : $Author:   tmortsol  $, U.S. Robotics Access Corp.
 *****************************************************************************
 *          Copyright (c) $Date:   May 13 1996 14:48:44  $,                  *
 *           U.S. Robotics Access Corp., Microsoft Corp.                     *
 *                       All Rights Reserved                                 *
 *****************************************************************************
 *                          "PPTP MODULE"                                    *
 * This software may be used to implement the POINT to POINT TUNNELING       *
 * PROTOCOL as defined by the PPTP forum.  The PPTP forum consists of:       *
 * U.S. Robotics, Microsoft, Ascend Communications, 3Com/Primary Access,     *
 * Telematics.                                                               *
 *****************************************************************************
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain this entire header
 *    including the above copyright notice, this list of conditions and
 *    the following disclaimer.
 * 2. The name of U.S. Robotics may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission of U.S. Robotics.
 *
 * THIS SOFTWARE IS PROVIDED BY U.S. ROBOTICS ACESS CORP. ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL U.S. ROBOTICS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*END_HEADER*/

/*
 * pptppro.h
 *
 * This header file contains PPTP protocol definitions.
 *
 * This file represents the PPTP protocol developed by the PPTP forum
 * from the PPTP Technical Specification dated October 25, 1995
 * and updates to this specification from PPTP meeting notes taken
 * on October 27, 1995
 */

#if !defined(_pptppro_h)
#define _pptppro_h

/*
 * If the PPTP message data types used by the message structure definitions
 * are not defined then define them
 */

#if !defined(PPTP_MESSAGE_DATA_TYPES)
#define PPTP_MESSAGE_DATA_TYPES

typedef unsigned char Byte;
typedef unsigned short Word;
typedef unsigned long LongWord;

typedef struct {
#if !defined(PPTP_BIG_ENDIAN)
	Byte				recursionControl:3,
					strictSourceRoutePresent:1,
					seqNumberPresent:1,
					keyPresent:1,
					routingPresent:1,
					checksumPresent:1;
	Byte				version:3,
					flags:4,
					ackPresent:1;
#else
	Byte				checksumPresent:1,
					routingPresent:1,
					keyPresent:1,
					seqNumberPresent:1,
					strictSourceRoutePresent:1,
					recursionControl:3;
	Byte				ackPresent:1,
					flags:4,
					version:3;
#endif /* !defined(PPTP_BIG_ENDIAN) */
	Word				protocolType;
	Word				keyLength;
	Word				keyID;
} StaticGreHeader;

typedef struct {
	StaticGreHeader		staticGreHeader;
	union {
		struct {
			LongWord			seqNumber;
			LongWord			ackNumber;
		} SeqPresent;
		LongWord			ackNumber;
	};
} PptpGreHeader;

#endif /* !defined(PPTP_MESSAGE_DATA_TYPES) */

/*
 * Begin defining constants used within the PPTP protocol
 */

/*
 * Well known TCP port number used for the PPTP control channel
 */

#define PPTP_TCP_PORT_NUMBER (5678)

/*
 * Well known protocol number used to tunnel PPTP data packets
 * over GRE
 */

#define PPTP_GRE_PPTP_PROTOCOL_NUMBER (0X5678)

/*
 * IP protocol number used by GRE
 */

#define PPTP_IP_GRE_PROTOCOL_NUMBER (47)

/*
 * Latest version number of the PPTP protocol this header file is
 * targeted at.  The high byte is the version number and the low
 * byte is the revision number within the version.
 * This header file supports version 1.0
 */

#define PPTP_PROTOCOL_VERSION (0X0100)

/*
 * The maximum MTU size of PPP packets transmitted and received through
 * the PPTP data channels
 */

#define PPTP_MAX_DATA_MTU          (1532)

/*
 * Magic cookie value used to synchronize messages sent within the
 * TCP control channel
 */

#define PPTP_MAGIC_COOKIE (0X1a2b3c4d)

/*
 * Default window size used for within the PPTP data channel
 */

#define PPTP_DEFAULT_PACKET_WINDOW (3)

/*
 * Default PPTP data channel flow control static timeout value in seconds
 */

#define PPTP_DEFAULT_DATA_CHANNEL_FLOW_CONTROL_STATIC_TIMEOUT (1)

/*
 * Max sequence number used within the PPTP data channel
 * Sequence numbers beyond this value will wraparound to zero
 */

#define PPTP_MAX_SEQUENCE_NUMBER (0XFFFFFFFF)

/*
 * Number of seconds of inactivity to wait before sending a PPTP_ECHO_REQUEST
 * message
 */

#define PPTP_DEFAULT_INACTIVITY_IDLE_SECONDS (60)

/*
 * Number of seconds to wait for a PPTP_ECHO_REPLY response to a
 * PPTP_ECHO_REQUEST before timing out the control connection
 */

#define PPTP_DEFAULT_ECHO_REPLY_TIMEOUT_SECONDS (60)

/*
 * Number of bytes reserved for the "hostname" field used in PPTP messages
 */

#define MAX_HOSTNAME_LENGTH (64)

/*
 * Number of bytes reseverd for the "vendorString" field used in PPTP messages
 */

#define MAX_VENDOR_STRING_LENGTH (64)

/*
 * Number of bytes reserved for all variants of call addresses
 * used in PPTP messages
 */

#define MAX_PHONE_NUMBER_LENGTH (64)

/*
 * Number of bytes reserved for the "callStatistics" field used in
 * PPTP messages
 */

#define MAX_CALL_STATS_LENGTH (128)

/*
 * Number of seconds to wait before a new PPTP_WAN_ERROR_NOTIFY message
 * can be sent to a server for a particular call
 */

#define PPTP_WAN_ERROR_NOTIFY_RETRANSMIT_SECONDS (60)

/*
 * PptpControlMessageType defines the different control message types
 * that will be transmitted via the PPTP control channel.
 * The descriptions of these messages is deferred for the message structure
 * definition of each message
 */

typedef enum {
	PPTP_START_SESSION_REQUEST = 1,
	PPTP_START_SESSION_REPLY,
	PPTP_STOP_SESSION_REQUEST,
	PPTP_STOP_SESSION_REPLY,
	PPTP_ECHO_REQUEST,
	PPTP_ECHO_REPLY,
	PPTP_OUT_CALL_REQUEST,
	PPTP_OUT_CALL_REPLY,
	PPTP_IN_CALL_REQUEST,
	PPTP_IN_CALL_REPLY,
	PPTP_IN_CALL_CONNECTED,
	PPTP_CALL_CLEAR_REQUEST,
	PPTP_CALL_DISCONNECT_NOTIFY,
	PPTP_WAN_ERROR_NOTIFY,
	PPTP_SET_LINK_INFO,
	PPTP_NUMBER_OF_CONTROL_MESSAGES
} PptpControlMessageType;

/*
 * PptpCallFramingType defines the PPP framing types sent within
 * PPTP call control messages
 *
 * PPTP_ASYNC_FRAMING			Represents async PPP framing
 *
 * PPTP_SYNC_FRAMING			Represents sync PPP framing
 *
 * PPTP_DONT_CARE_FRAMING_TYPE		Represents nonspecified PPP framing
 */

typedef enum {
	PPTP_ASYNC_FRAMING = 1,
	PPTP_SYNC_FRAMING,
	PPTP_DONT_CARE_FRAMING
} PptpFramingType;

/*
 * PptpCallBearerType defines the call bearer types sent within
 * PPTP call control messages
 *
 * PPTP_ANALOG_CALL			Represents an analog call bearer
 *
 * PPTP_DIGITAL_CALL			Represents a digital call bearer
 *
 * PPTP_DONT_CARE_BEARER_TYPE		Represents a nonspecified call
 *					bearer type
 */

typedef enum {
	PPTP_ANALOG_CALL = 1,
	PPTP_DIGIGTAL_CALL,
	PPTP_DONT_CARE
} PptpCallBearerType;


/*
 * Constants used as bit fields to define the framing capabilites
 * of a sender of a PPTP_START_SESSION_REQUEST or PPTP_START_SESSION_REPLY
 * message.  These capabilities are or'd together and stored in the
 * framingCapability of the message
 *
 * PPTP_FRAME_CAP_ASYNC			Able to do async PPP framing
 *
 * PPTP_FRAME_CAP_SYNC			Able to do sync PPP framing
 */

#define PPTP_FRAME_CAP_ASYNC		(0X00000001L)
#define PPTP_FRAME_CAP_SYNC		(0X00000002L)

/*
 * Constans used as bit fields to define the call bearer capabilities
 * of a sender of a PPTP_START_SESSION_REQUEST or PPTP_START_SESSION_REPLY
 * message.  These capabilities are or'd together and stored in the
 * bearerCapability of the message
 *
 * PPTP_BEARER_CAP_ANALOG		Able to make analog calls
 *
 * PPTP_BEARER_CAP_DIGITAL		Able to make digital calls
 */

#define PPTP_BEARER_CAP_ANALOG		(0X00000001L)
#define PPTP_BEARER_CAP_DIGITAL		(0X00000002L)

/*
 * PptpGeneralError defines the different general result value that is
 * contained in every response message.  Each PPTP control response message
 * contains both a resultCode and generalErrorCode field.  General result
 * codes are result values that are common between many different message
 * types while the specific result codes are result values that are unique
 * to the specific operation.  If the resultCode indicates that a general
 * error has occurred then the result is stored in the generalErrorCode
 * field.
 *
 * PPTP_ERROR_CODE_NONE			No general error has occurred
 *
 * PPTP_NOT_CONNECTED			A PPTP control session
 *					has not yet been established
 *
 * PPTP_BAD_FORMAT			The request packet had a
 *					invalid message length
 *
 * PPTP_BAD_VALUE			The request packet had
 *					a bad value
 *
 * PPTP_NO_RESOURCE			There are no resources to
 *					handle the request at this
 *					time
 *
 * PPTP_INVALID_STATE			The request ocurred during
 *					an invalid state
 *
 * PPTP_REMOTE_DEVICE_ERROR		A vendor specific error
 *					has occurred on a remote
 *					device
 */

typedef enum {
	PPTP_ERROR_CODE_NONE = 0,
	PPTP_NOT_CONNECTED,
	PPTP_BAD_FORMAT,
	PPTP_BAD_VALUE,
	PPTP_NO_RESOURCE,
	PPTP_BAD_CALLID,
	PPTP_REMOTE_DEVICE_ERROR
} PptpGeneralError;

/*
 * Begin defining the PPTP control messages.  For each message structure there
 * are 2 different stuctures defined that can be used to represent the
 * message.  The first structure will match the exact format within the
 * PPTP specification and represents message structures that have the
 * data elements within the message but do not contain the control header.
 * The second definition, referred to as expanded definitions,
 * is a definition that contains the control header followed by the
 * data elements within the message.  The nomenclature used for these
 * expanded message structures is the name of the first message structure
 * followed by a "Msg" suffix.
 */

/*
 * PptpPacketType defines the two types of PPTP control channel packets:
 *
 * PPTP_CONTROL_PACKET			The packet contains PPTP control
 *					messages
 *
 * PPTP_MGMT_PACKET			The packet contains remote management
 *					data.  The format of these packets
 *					is currently undefined
 */

typedef enum {
	PPTP_CONTROL_PACKET = 1,
	PPTP_MGMT_PACKET
} PptpPacketType;

/*
 * PptpPacketHeader is the control packet header describes the contents
 * of a PPCP packet sent within a PPTP control channel.  This curently
 * includes both control message and management data packets
 *
 * packetLength				Total length of packet including
 *					this header
 *
 * packetType				Value of PptpPacketType
 *
 * magicCookie				Used to synchronize the packets
 *					sent within the control channel
 *					in case the endpoints unsynchronize.
 *					Must be equal to PPTP_MAGIC_COOKIE
 */

typedef struct {
	Word		packetLength;
	Word		packetType;
	LongWord	magicCookie;
} PptpPacketHeader;

/*
 * PptpControlHeader is the specific control message header added to
 * a control message that further identifies the type of control message
 * sent in a PPTP_CONTROL_PACKET.
 *
 * messageType				Value of PptpControlMessageType
 *
 * reserved				Currently used for padding
 */

typedef struct {
	Word		messageType;
	Word		reserved;
} PptpControlHeader;

/*
 * PptpControlMessageHeader is an expanded definition of the complete
 * header used for a PPTP control message packet.  It is referenced
 * as the header by all the expanded PPTP control messages
 *
 * packetLength				Total length of packet including
 *					this header
 *
 * packetType				Value of PptpPacketType
 *
 * magicCookie				Used to synchronize the packets
 *					sent within the control channel
 *					in case the endpoints unsynchronize.
 *					Must be equal to PPTP_MAGIC_COOKIE
 *
 * messageType				Value of PptpControlMessageType
 *
 * reserved				Currently used for padding
 */

typedef struct {
	Word		packetLength;
	Word		packetType;
	LongWord	magicCookie;
	Word		messageType;
	Word		reserved;
} PptpControlMessageHeader;

/*
 * PptpFepInfo is a common structure used to define a FEP in both the
 * PPTP_START_SESSION_REQUEST and PPTP_STOP_SESSION_REPLY messages.
 * This structure is used in the expanded PPTP message structures for these
 * messages
 */

typedef struct {
	LongWord	framingCapability;
	LongWord	bearerCapability;
	Word		maxChannels;
	Word		firmwareRevision;
	Byte		hostName[MAX_HOSTNAME_LENGTH];
	Byte		vendorString[MAX_VENDOR_STRING_LENGTH];
} PptpFepInfo;

/*
 * PptpStartSessionRequest
 *
 * This message is sent by a server or FEP to initiate a PPTP control
 * channel with the remote peer
 *
 * protocolVersion			The PPTP protocol version
 *					supported by the sender
 *
 * reserved1				Currently used for padding
 *
 * reserved2				Currently used for padding
 *
 * framingCapability			The type of PPP packet framing
 *					supported by the sender
 *
 * bearerCapability			The call bearer capabilities
 *					supported by the sender
 *
 * maxChannels				The maximum number of PPTP calls
 *					the sender can handle
 *
 * hostname				ASCII Host name of sender
 *
 * vendorString				Vendor specific ASCII string
 *					that describes the sender
 */

typedef struct {
	Word				protocolVersion;
	Byte				reserved1;
	Byte				reserved2;
	LongWord			framingCapability;
	LongWord			bearerCapability;
	Word				maxChannels;
	Word				firmwareRevision;
	Byte				hostName[MAX_HOSTNAME_LENGTH];
	Byte				vendorString[MAX_VENDOR_STRING_LENGTH];
} PptpStartSessionRequest;

/*
 * Expanded message structure for PPTP_START_SESSION_REQUEST
 */

typedef struct {
	PptpControlMessageHeader	header;
	Word				protocolVersion;
	Byte				reserved1;
	Byte				reserved2;
	PptpFepInfo			fepInfo;
} PptpStartSessionRequestMsg;

/*
 * PptpStartSessionResultCode contains the specific result codes for the
 * resultCode field in the PPTP_START_SESSION_REPLY message
 *
 * PPTP_START_OK			Control channel successfullly
 *					established
 *
 * PPTP_START_GENERAL_ERROR		A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 *
 * PPTP_START_ALREADY_CONNECTED		The remote peer already has
 *					a control channel established
 *					with the sender
 *
 * PPTP_START_NOT_AUTHORIZED		The remote peer is not authorized
 *					to talk to the sender
 *
 * PPTP_START_UNKNOWN_PROTOCOL		The host cannot negotiate the PPTP
 *					version number that was requested
 */

typedef enum {
	PPTP_START_OK = 1,
	PPTP_START_GENERAL_ERROR,
	PPTP_START_ALREADY_CONNECTED,
	PPTP_START_NOT_AUTHORIZED,
	PPTP_START_UNKNOWN_PROTOCOL
} PptpStartSessionResultCode;

/*
 * PptpStartSessionReply
 *
 * This message is sent in response to a PPTP_START_SESSION_REQUEST
 *
 * protocolVersion			The PPTP protocol version
 *					supported by the sender
 *
 * resultCode				Value of PptpStartSessionResultCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 * framingCapability			The type of PPP packet framing
 *					supported by the sender
 *
 * bearerCapability			The call bearer capabilities
 *					supported by the sender
 *
 * maxChannels				The maximum number of PPTP calls
 *					the sender can handle
 *
 * hostname				ASCII Host name of sender
 *
 * vendorString				Vendor specific ASCII string
 *					that describes the sender
 */

typedef struct {
	Word				protocolVersion;
	Byte				resultCode;
	Byte				generalErrorCode;
	LongWord			framingCapability;
	LongWord			bearerCapability;
	Word				maxChannels;
	Word				firmwareRevision;
	Byte				hostName[MAX_HOSTNAME_LENGTH];
	Byte				vendorString[MAX_VENDOR_STRING_LENGTH];
} PptpStartSessionReply;

/*
 * Expanded message structure for PPPT_START_SESSION_REPLY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				protocolVersion;
	Byte				resultCode;
	Byte				generalErrorCode;
	PptpFepInfo			fepInfo;
} PptpStartSessionReplyMsg;

/*
 * PptpStopReasons contains the reason the sender is requesting termination
 * of the control channel.
 *
 * PPTP_STOP_NONE			Normal control session teardown
 *
 * PPTP_STOP_PROTOCOL			The protocol version indicated in the
 *					PptpStartSessionReply cannot be
 *					supported
 *
 * PPTP_STOP_LOCAL_SHUTDOWN		An administrative shutdown has
 *					occurred
 */

typedef enum {
	PPTP_STOP_NONE = 1,
	PPTP_STOP_PROTOCOL,
	PPTP_STOP_LOCAL_SHUTDOWN
} PptpStopReasons;

/*
 * PptpStopSessionRequest
 *
 * This message is sent to request teardown of an established control
 * channel.  It will cause all outstanding calls to passively terminate
 * before closing the control channel (i.e. no disconnet notifications will
 * be sent)
 *
 * reason				Value of PptpStopReasons
 *
 */

typedef struct {
	Byte	reason;
	Byte	reserved1;
	Word	reserved2;
} PptpStopSessionRequest;

/*
 * Expanded message structure for PPPT_STOP_SESSION_REQUEST
 */

typedef struct {
	PptpControlMessageHeader	header;
	Byte						reason;
	Byte						reserved1;
	Word						reserved2;
} PptpStopSessionRequestMsg;

/*
 * PptpStopSessionResultCode contains the specific result codes for the
 * resultCode field in the PPTP_STOP_SESSION_REPLY message
 *
 * PPTP_STOP_OK				The control channel was
 *					successfully torn down
 *
 * PPTP_START_GENERAL_ERROR		A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 */

typedef enum {
  	PPTP_STOP_OK = 1,
	PPTP_STOP_GENERAL_ERROR
} PptpStopSessionResultCode;

/*
 * PptpStopSessionReply
 *
 * This message is sent in response to a PPTP_START_SESSION_REQUEST
 *
 * resultCode				Value of PptpStopSessionResultCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 */

typedef struct {
	Byte	resultCode;
	Byte	generalErrorCode;
	Word	reserved1;
} PptpStopSessionReply;

typedef struct {
	PptpControlMessageHeader 	header;
	Byte						resultCode;
	Byte						generalErrorCode;
	Word						reserved1;
} PptpStopSessionReplyMsg;

/*
 * PptpEchoRequest
 *
 * This message is sent by a peer during periods of inactivity to
 * verify that a control channel is still established with the remote
 * peer.  This message should be sent after PPTP_ECHO_ACTIVITY_SECONDS
 * seconds of inactivity by both peers.  A peer should timeout a remote
 * host if a reply has not within PPTP_ECHO_REPLY_TIMEOUT_SECONDS of sending
 * this message.
 *
 * identNumber				Sequence number this echo request
 *					assigned by sender to correlate
 *					the reply to the request
 */

typedef struct {
	LongWord			identNumber;
} PptpEchoRequest;

/*
 * Expanded message structure for PPPT_STOP_SESSION_REQUEST
 */

typedef struct {
	PptpControlMessageHeader 	header;
	LongWord			identNumber;
} PptpEchoRequestMsg;

/*
 * PptpEchoReplyResultCode contains the specific result codes for the
 * resultCode field in the PPTP_ECHO_REPLY message
 *
 * PPTP_ECHO_OK				The echo reply is valid
 *
 * PPTP_ECHO_GENERAL_ERROR		A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 */

typedef enum {
	PPTP_ECHO_OK = 1,
	PPTP_ECHO_GENERAL_ERROR
} PptpEchoReplyResultCode;

/*
 * PptpEchoReply
 *
 * This message is sent in response to a PPTP_ECHO_REPLY
 *
 * identNumber				Sequence number that matches the
 *					corresponding echo request message
 *
 * resultCode				Value of PptpEchoReplyResultCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 * reserved				Currently used for padding
 */

typedef struct {
	LongWord			identNumber;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				reserved;
} PptpEchoReply;

/*
 * Expanded message structure for PPPT_ECHO_REPLY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	LongWord			identNumber;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				reserved;
} PptpEchoReplyMsg;

/*
 * PptpOutCallRequest
 *
 * This message is sent from a server to initiate an outbound call
 *
 * callID				Unique ID assigned by sender of
 *					this message to the sender will
 *					use to track the call.  All further
 *					messages sent by a remote peer for
 *					this call must use contain this
 *					identifier
 *
 * callSerialNumber			Serial number used to track the
 *					call for debugging purposes
 *
 * minBPS				Mininum acceptable baud rate to
 *					connect at
 *
 * maxBPS				Maximum acceptable baud rate to
 *					connect at
 *
 * bearerType				The call bearer type that should
 *					be used to establish the call.
 *					Value of PptpCallBearerType
 *
 * framingType				PPP framing type used for the PPP
 *					data.  Value of PptpFramingType
 *
 * packetWindow				RX packet window used by sender
 *
 * packetProcDelay			Packet processing delay for sender
 *					in tenths of a second
 *
 * phoneNumberLength			Length of number in phoneNumber
 *					parameter
 *
 * reserved				Currently used for padding
 *
 * phoneNumber				Number to dial
 *
 * subAddress				ISDN vendor specific field
 */

typedef struct {
	Word				callID;
	Word				callSerialNumber;
	LongWord			minBPS;
	LongWord			maxBPS;
	LongWord			bearerType;
	LongWord			framingType;
	Word				packetWindow;
	Word				packetProcDelay;
	Word				phoneNumberLength;
	Word				reserved;
	Byte				phoneNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				subAddress[MAX_PHONE_NUMBER_LENGTH];
} PptpOutCallRequest;

/*
 * Expanded message structure for PPPT_OUT_CALL_REQUEST
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Word				callSerialNumber;
	LongWord			minBPS;
	LongWord			maxBPS;
	LongWord			bearerType;
	LongWord			framingType;
	Word				packetWindow;
	Word				packetProcDelay;
	Word				phoneNumberLength;
	Word				reserved;
	Byte				phoneNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				subAddress[MAX_PHONE_NUMBER_LENGTH];
} PptpOutCallRequestMsg;

/*
 * PptpOutCallResultCode contains the specific result codes for the
 * resultCode field in the PPTP_OUT_CALL_REPLY message
 *
 * PPTP_OUT_CALL_CONNECT		The call has been established
 *
 * PPTP_OUT_CALL_GENERAL_ERROR		A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 *
 * PPTP_OUT_CALL_NO_CARRIER		No carrier could be detected with
 *					the remote end
 *
 * PPTP_OUT_CALL_BUSY			Line to remote end is busy
 *
 * PPTP_OUT_CALL_NO_DIAL_TONE		No dial tone detected at local end
 *
 * PPTP_OUT_CALL_TIMEOUT		The call was not established in time
 *
 * PPTP_OUT_CALL_DONT_ACCEPT            Do not accept the call
 *
 * PPTP_OUT_CALL_ABORTED                The call was aborted
 */

typedef enum {
	PPTP_OUT_CALL_CONNECT = 1,
	PPTP_OUT_CALL_GENERAL_ERROR,
	PPTP_OUT_CALL_NO_CARRIER,
	PPTP_OUT_CALL_BUSY,
	PPTP_OUT_CALL_NO_DIAL_TONE,
	PPTP_OUT_CALL_TIMEOUT,
	PPTP_OUT_CALL_DONT_ACCEPT,
	PPTP_OUT_CALL_ABORTED
} PptpOutCallResultCode;

/*
 * PptpOutCallReply
 *
 * This message is sent in response to a PPTP_OUT_CALL_REQUEST message
 * once the status of the call has been determined.
 *
 * callID				Unique ID assigned by sender of
 *					this message to the sender will
 *					use to track the call.  All further
 *					messages sent by a remote peer for
 *					this call must use contain this
 *					identifier
 *
 * peerCallID				Call ID value used by peer that
 *					began this call.  This field is
 *					filled in with the callID field
 *					from the PptpOutCallRequest message
 *
 * resultCode				Value of PptpOutCallResultCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 * causeCode				Vendor specific code that is
 *					specific to call type
 *
 * connectSpeed				Actual BPS that call connected at
 *
 * packetWindow				RX packet window used by sender
 *
 * packetProcDelay			Packet processing delay for sender
 *					in tenths of a second
 *
 * physChannelID:			physical line number FEP used for call
 *					(used for debugging purposes)
 *					This field is vendor specific
 */

typedef struct {
	Word				callID;
	Word				peerCallID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				causeCode;
	LongWord			connectSpeed;
	Word				packetWindow;
	Word				packetProcDelay;
	LongWord			physChannelID;
} PptpOutCallReply;

/*
 * Expanded message structure for PPPT_OUT_CALL_REPLY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Word				peerCallID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				causeCode;
	LongWord			connectSpeed;
	Word				packetWindow;
	Word				packetProcDelay;
	LongWord			physChannelID;
} PptpOutCallReplyMsg;

/*
 * PptpInCallRequest
 *
 * This message is sent to a server when a new incoming call has arrived
 *
 * callID				Unique ID assigned by sender of
 *					this message to the sender will
 *					use to track the call.  All further
 *					messages sent by a remote peer for
 *					this call must use contain this
 *					identifier
 *
 * callSerialNumber			Serial number used to track the
 *					call for debugging purposes
 *
 * bearerType				The call bearer type that should
 *					be used to establish the call.
 *					Value of PptpCallBearerType
 *
 * physChannelID:			physical line number FEP used for call
 *					(used for debugging purposes)
 *					This field is vendor specific
 *
 * dialedNumberLength			Length of number in dialedNumber
 *					parameter
 *
 * dialingNumberLength			Length of number to dialingNumber
 *					parameter
 *
 * dialedNumber				Number to dial
 *
 * dialingNumber			Number to dial
 *
 * subAddress				ISDN vendor specific field
 */

typedef struct {
	Word				callID;
	Word				callSerialNumber;
	LongWord			bearerType;
	LongWord			physChannelID;
	Word				dialedNumberLength;
	Word				dialingNumberLength;
	Byte				dialedNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				dialingNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				subAddress[MAX_PHONE_NUMBER_LENGTH];
} PptpInCallRequest;

/*
 * Expanded message structure for PPTP_IN_CALL_REQUEST
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Word				callSerialNumber;
	LongWord			bearerType;
	LongWord			physChannelID;
	Word				dialedNumberLength;
	Word				dialingNumberLength;
	Byte				dialedNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				dialingNumber[MAX_PHONE_NUMBER_LENGTH];
	Byte				subAddress[MAX_PHONE_NUMBER_LENGTH];
} PptpInCallRequestMsg;

/*
 * PptpInCallResultCode
 *
 * PptpInCallResultCode contains the specific result codes for the
 * resultCode field in the PPTP_IN_CALL_REPLY message
 *
 * PPTP_IN_CALL_ACCEPT			The call has been established
 *
 * PPTP_IN_CALL_GENERAL_ERROR		A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 *
 * PPTP_IN_CALL_DONT_ACCEPT		The call is not accepted by the server
 *
 * PPTP_OUT_CALL_ABORTED                The call was aborted
 */

typedef enum {
	PPTP_IN_CALL_ACCEPT = 1,
	PPTP_IN_CALL_GENERAL_ERROR,
	PPTP_IN_CALL_DONT_ACCEPT,
	PPTP_IN_CALL_ABORTED
} PptpInCallResultCode;

/*
 * PptpInCallReply
 *
 * This message is sent in response to a PPTP_IN_CALL_REQUEST message
 * informing the client whether or not the call should be answered
 *
 * callID				Unique ID assigned by sender of
 *					this message to the sender will
 *					use to track the call.  All further
 *					messages sent by a remote peer for
 *					this call must use contain this
 *					identifier
 *
 * peerCallID				Call ID value used by peer that
 *					began this call.  This field is
 *					filled in with the callID field
 *					from the PptpInCallRequest message
 *
 * resultCode				Value of PptpInCallResultCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 * packetWindow				RX packet window used by sender
 *
 * packetProcDelay			Packet processing delay for sender
 *					in tenths of a second
 *
 * reserved				Currently used for padding
 */

typedef struct {
	Word				callID;
	Word				peerCallID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				packetWindow;
	Word				packetProcDelay;
	Word				reserved;
} PptpInCallReply;

/*
 * Expanded message structure for PPTP_IN_CALL_REPLY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Word				peerCallID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				packetWindow;
	Word				packetProcDelay;
	Word				reserved;
} PptpInCallReplyMsg;

/*
 * PptpInCallConnected
 *
 * This message is sent to a server in response to a PPTP_IN_CALL_REPLY
 * message once the status of the call has been determined
 *
 * peerCallID		Call ID value that was in the
 *					callID field in the Incoming Call
 *					Reply message
 *
 * connectSpeed		Actual BPS that call connected at
 *
 * packetWindow		RX packet window used by sender
 *
 * packetProcDelay	Packet processing delay for sender
 *					in tenths of a second
 *
 * framingType		PPP framing type used for the PPP
 *					data.  Value of PptpFramingType
 */

typedef struct {
	Word				peerCallID;
	Word				reserved1;
	LongWord			connectSpeed;
	Word				packetWindow;
	Word				packetProcDelay;
	LongWord			framingType;
} PptpInCallConnected;

/*
 * Expanded message structure for PPTP_IN_CALL_CONNECTED
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				peerCallID;
	Word				reserved1;
	LongWord			connectSpeed;
	Word				packetWindow;
	Word				packetProcDelay;
	LongWord			framingType;
} PptpInCallConnectedMsg;

/*
 * PptpCallClearRequest
 *
 * This message is sent by a server to request a disconnect of a connected
 * call
 *
 * callID				Call ID value used by originator.
 *					This field is used instead of the
 *					peer call ID because the value
 *					of the peer call ID may not
 *					be known during call establishment
 *
 * reserved				Currently used for padding
 */

typedef struct {
	Word				callID;
	Word				reserved;
} PptpCallClearRequest;

/*
 * Expanded message structure for PPTP_CLEAR_CALL_REQUEST
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Word				reserved;
} PptpCallClearRequestMsg;

/*
 * PptpCallDisconnectCode contains the specific result codes for the
 * resultCode field in the PPTP_CALL_DISCONNECT_NOTIFY message
 *
 * PPTP_DISCONNECT_LOST_CARRIER		Carrier was lost to the
 *					remote endpoint
 *
 * PPTP_DISCONNECT_GENERAL_ERROR	A general error has occurred, look
 *					at the generalErrorCode field to
 *					determine the error
 *
 * PPTP_DISCONNECT_LOCAL_SHUTDOWN	An administrator has shutdown
 *					the connection
 */

typedef enum {
	PPTP_DISCONNECT_LOST_CARRIER = 1,
	PPTP_DISCONNECT_GENERAL_ERROR,
	PPTP_DISCONNECT_ADMIN_SHUTDOWN
} PptpCallDisconnectCode;

/*
 * PptpCallDisconnectNotify
 *
 * This message is sent to a server to notify the server of a disconnected
 * call
 *
 * callID				Call ID value used by originator.
 *					This field is used instead of the
 *					peer call ID because the value
 *					of the peer call ID may not
 *					be known during call establishment
 *
 * resultCode				Value of PptpCallDisconnectCode
 *
 * generalErrorCode			Value of PptpGeneralError
 *
 * causeCode				Vendor specific code that is
 *					specific to call type
 *
 * reserved				Currently used for padding
 *
 * callStatistics			A vendor specific string containing
 *					call statistics information that
 *					can be logged for debugging purposes.
 *					This value is a null terminated
 *					ASCII string
 */

typedef struct {
	Word				callID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				causeCode;
	Word				reserved;
	Byte				callStatistics[MAX_CALL_STATS_LENGTH];
} PptpCallDisconnectNotify;

/*
 * Expanded message structure for PPTP_CALL_DISCONNECT_NOTIFY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				callID;
	Byte				resultCode;
	Byte				generalErrorCode;
	Word				causeCode;
	Word				reserved;
	Byte				callStatistics[MAX_CALL_STATS_LENGTH];
} PptpCallDisconnectNotifyMsg;

/*
 * PptpWanErrorNotify
 *
 * This message is sent to a server to notify the server of any
 * line hardware or datalink errors.  All counter fields in this
 * message are cumulative for a particular call.  This message should be sent
 * when one of the error events in this message occurs but should be sent
 * not more than once every PPTP_WAN_ERROR_NOTIFY_RETRANSMIT_SECONDS
 *
 * peerCallID				Call ID value used by peer.
 *					This field is filled in with
 *					the callID field from the
 *					message sent by the peer
 *					during call establishment
 *
 * reserved				Currently used for padding
 *
 * crcErrors				Number of CRC errors
 *
 * framingErrors			Number of framing errors
 *
 * hardwareOverruns			Number of hardware errors
 *
 * timeoutErrors			Number of timeout errors
 *
 * alignmentErrors			Number of alignment errors
 */

typedef struct {
	Word				peerCallID;
	Word				reserved;
	LongWord			crcErrors;
	LongWord			framingErrors;
	LongWord			hardwareOverRuns;
	LongWord			bufferOverRuns;
	LongWord			timeoutErrors;
	LongWord			alignmentErrors;
} PptpWanErrorNotify;

/*
 * Expanded message structure for PPTP_WAN_ERROR_NOTIFY
 */

typedef struct {
	PptpControlMessageHeader 	header;
	Word				peerCallID;
	Word				reserved;
	LongWord			crcErrors;
	LongWord			framingErrors;
	LongWord			hardwareOverRuns;
	LongWord			bufferOverRuns;
	LongWord			timeoutErrors;
	LongWord			alignmentErrors;
} PptpWanErrorNotifyMsg;

/*
 * PptpSetLinkInfo
 *
 * This message is sent by the server to set the PPP LCP
 * options processing that should be done by a client so
 * it can be offloaded from the server to obtain better
 * PPP throughput
 *
 * peerCallID				Call ID value used by peer.
 *					This field is filled in with
 *					the callID field from the
 *					message sent by the peer
 *					during call establishment
 *
 * reserved				Currently used for padding
 *
 * sendAccm				Send ACCM value the client should use
 *					to process outgoing PPP packets.
 *				        The default value
 *					used by the client until this
 *					message is received is 0XFFFFFFFF
 *
 * recvAccm				Recv ACCM value the client should use
 *					to process incoming PPP packets.
 *					The default value
 *					used by the client until this
 *					message is received is 0XFFFFFFFF
 */

typedef struct {
	Word				peerCallID;
	Word				reserved;
	LongWord			sendAccm;
	LongWord			recvAccm;
} PptpSetLinkInfo;

/*
 * Expanded message structure for PPTP_SET_LINK_INFO
 */

typedef struct {
	PptpControlMessageHeader	header;
	Word				peerCallID;
	Word				reserved;
	LongWord			sendAccm;
	LongWord			recvAccm;
} PptpSetLinkInfoMsg;

/*
 * Pptp control packet is a union of all the PPTP control messages in
 * expanded form
 */

typedef union {
	PptpControlMessageHeader	header;
	PptpStartSessionRequestMsg	startSessionRequest;
	PptpStartSessionReplyMsg	startSessionReply;
	PptpStopSessionRequestMsg	stopSessionRequest;
	PptpStopSessionReplyMsg		stopSessionReply;
	PptpEchoRequestMsg		echoRequest;
	PptpEchoReplyMsg		echoReply;
	PptpOutCallRequestMsg		outCallRequest;
	PptpOutCallReplyMsg		outCallReply;
	PptpInCallRequestMsg		inCallRequest;
	PptpInCallReplyMsg		inCallReply;
	PptpInCallConnectedMsg		inCallConnected;
	PptpCallClearRequestMsg		callClearRequest;
	PptpCallDisconnectNotifyMsg	callDisconnectNotify;
	PptpWanErrorNotifyMsg		wanErrorNotify;
	PptpSetLinkInfoMsg		setLinkInfo;
} PptpControlPacket;

/*
 * Pptp data packet defines the format of a PPTP data packet
 * Note that this structure is byte order dependent.
 */

typedef struct {
	PptpGreHeader			greHeader;
	Byte				data[PPTP_MAX_DATA_MTU];
} PptpDataPacket;

#endif /* !defined(_pptppro_h) */
